#include <windows.h>
#include <d3d8.h>
#include <d3dx8.h>
#include <stdio.h>
#include <commdlg.h>
#include <commctrl.h>
#include "d3dsaver.h"
#include "d3dfont.h"
#include "Jeresaver.h"
#include "Resource.h"
#include "d3dutil.h"
#include "dxutil.h"




Jeresaver* g_pMyMoireScreensaver = NULL;




// Helper function to stuff a FLOAT into a DWORD argument
inline DWORD FtoDW( FLOAT f ) { return *((DWORD*)&f); }

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    HRESULT hr;
    Jeresaver saver;

    if( FAILED( hr = saver.Create( hInst ) ) )
    {
        saver.DisplayErrorMsg( hr );
        return 0;
    }

    return saver.Run();
}




//-----------------------------------------------------------------------------
// Name: Jeresaver()
// Desc: Constructor
//-----------------------------------------------------------------------------
Jeresaver::Jeresaver( )
{
    g_pMyMoireScreensaver = this;

    InitCommonControls();

    ZeroMemory( m_DeviceObjectsArray, sizeof(m_DeviceObjectsArray) );

    lstrcpy( m_strRegPath, TEXT("Software\\Microsoft\\Screensavers\\Jeresaver") );
    LoadString( NULL, IDS_DESCRIPTION, m_strWindowTitle, 200 );

	myPointsTotal = 50000;

	myCamAngle = D3DXVECTOR3( 0, 0, 0 );
	D3DXMatrixIdentity( &myCamMat );

	myTextAngle = 0;
	myTextCenter = D3DXVECTOR3( 0, 0, 0 );
	myTextCameraZ = 0;

	myChangeTime = DXUtil_Timer( TIMER_GETABSOLUTETIME );

	// Settings
	mySetCreateSpeed = 1000;
	mySetRotateText = "XMunkki";
	mySetChangeInterval = 30;

	// Base settings
	m_bUseDepthBuffer = TRUE;
}




//-----------------------------------------------------------------------------
// Name: Randomize()
// Desc: Find a random number between 0 and iMax that is different from the
//       initial value of *piNum, and store it in *piNum.
//-----------------------------------------------------------------------------
VOID Jeresaver::Randomize( INT* piNum, INT iMax )
{
    INT iInit = *piNum;
    while ( *piNum == iInit )
    {
        *piNum = rand() % iMax;
    }
}




//-----------------------------------------------------------------------------
// Name: SetDevice()
// Desc: 
//-----------------------------------------------------------------------------
VOID Jeresaver::SetDevice( UINT iDevice )
{
    m_pDeviceObjects = &m_DeviceObjectsArray[iDevice];
}




//-----------------------------------------------------------------------------
// Name: BuildMeshes()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::BuildMeshes()
{
    SAFE_RELEASE( m_pDeviceObjects->m_pVBArray[0] );
	SAFE_RELEASE( m_pDeviceObjects->m_pTextMesh );

	// The point buffer
	m_pDeviceObjects->myPointsUsed = 0;
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( myPointsTotal * sizeof( FractalVertex ),
        D3DUSAGE_WRITEONLY, D3DFVF_FRACTALVERTEX, D3DPOOL_MANAGED, &m_pDeviceObjects->m_pVBArray[0] ) ) )
    {
        return E_FAIL;
    }

	// The text to rotate
	{
		HDC hdc = CreateCompatibleDC( NULL );
		HFONT hFont;
		HFONT hFontOld;
		LOGFONT lf;

		ZeroMemory( &lf, sizeof( lf ) );
		lf.lfHeight = 35;
		lf.lfStrikeOut = 1;
		lstrcpy( lf.lfFaceName, _T( "Arial" ) );

		hFont = CreateFontIndirect( &lf );
		hFontOld = ( HFONT ) SelectObject( hdc, hFont ); 
		if( FAILED( D3DXCreateText( m_pd3dDevice, hdc, mySetRotateText.c_str(),
			0.001f, 0.4f, &m_pDeviceObjects->m_pTextMesh, NULL, NULL) ) )
		{
			m_pDeviceObjects->m_pTextMesh = NULL;
/*			SelectObject( hdc, hFontOld );
			return E_FAIL;*/
		}
		SelectObject( hdc, hFontOld );
	}

	// Get the bounds for the text
	if ( m_pDeviceObjects->m_pTextMesh != NULL )
	{
		DWORD numVert = m_pDeviceObjects->m_pTextMesh->GetNumVertices();
		DWORD fvf = m_pDeviceObjects->m_pTextMesh->GetFVF();
		BYTE *data;

		// Get the bounding box
		if ( !FAILED( m_pDeviceObjects->m_pTextMesh->LockVertexBuffer( D3DLOCK_READONLY, &data ) ) )
		{
			D3DXVECTOR3 min, max;
			D3DXComputeBoundingBox( data, numVert, fvf, &min, &max );

			m_pDeviceObjects->m_pTextMesh->UnlockVertexBuffer();

			// Get the center
			myTextCenter.x = ( min.x + max.x ) / 2.f;
			myTextCenter.y = ( min.y + max.y ) / 2.f;
			myTextCenter.z = ( min.z + max.z ) / 2.f;

			// The bigger the text, move it further
			myTextCameraZ = -( max.x - min.x );
			if ( myTextCameraZ > -5.f )
				myTextCameraZ = -5.f;
		}
	}

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: BuildTextures()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::BuildTextures()
{
    HRESULT hr;

    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[0] );
/*    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[1] );
    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[2] );
    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[3] );*/

    if( FAILED( hr = LoadDDSTextureFromResource( m_pd3dDevice, 
        MAKEINTRESOURCE(IDR_BALL_DDS), &m_pDeviceObjects->m_pTexArray[0] ) ) )
    {
        return hr;
    }

/*    if( FAILED( hr = LoadDDSTextureFromResource( m_pd3dDevice, 
        MAKEINTRESOURCE(IDR_BALL_DDS), &m_pDeviceObjects->m_pTexArray[1] ) ) )
    {
        return hr;
    }

    if( FAILED( hr = LoadDDSTextureFromResource( m_pd3dDevice, 
        MAKEINTRESOURCE(IDR_NOISE_DDS), &m_pDeviceObjects->m_pTexArray[2] ) ) )
    {
        return hr;
    }

    if( FAILED( hr = LoadDDSTextureFromResource( m_pd3dDevice, 
        MAKEINTRESOURCE(IDR_SPIRAL_DDS), &m_pDeviceObjects->m_pTexArray[3] ) ) )
    {
        return hr;
    }*/

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: LoadDDSTextureFromResource()
// Desc: Note that this function asks D3DX to not create a full mip chain
//       for the texture.  Modify the NumMips parameter to the
//       D3DXCreateTextureFromFileInMemoryEx call if you want to reuse this
//       function elsewhere and you do want a full mip chain for the texture.
//-----------------------------------------------------------------------------
HRESULT Jeresaver::LoadDDSTextureFromResource( LPDIRECT3DDEVICE8 pd3dDevice, 
    TCHAR* strRes, LPDIRECT3DTEXTURE8* ppTex )
{
    HRESULT hr;
    HMODULE hModule = NULL;
    HRSRC rsrc;
    HGLOBAL hgData;
    LPVOID pvData;
    DWORD cbData;

    rsrc = FindResource( hModule, strRes, "DDS" );
    if( rsrc != NULL )
    {
        cbData = SizeofResource( hModule, rsrc );
        if( cbData > 0 )
        {
            hgData = LoadResource( hModule, rsrc );
            if( hgData != NULL )
            {
                pvData = LockResource( hgData );
                if( pvData != NULL )
                {
                    if( FAILED( hr = D3DXCreateTextureFromFileInMemoryEx( 
                        m_pd3dDevice, pvData, cbData, D3DX_DEFAULT, D3DX_DEFAULT, 
                        1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_NONE, 
                        D3DX_FILTER_NONE, 0, NULL, NULL, ppTex ) ) )
                    {
                        return hr;
                    }
                }
            }
        }
    }
    
    if( *ppTex == NULL)
        return E_FAIL;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT Jeresaver::FrameMove()
{
	// Rotate the camera
	myCamAngle.x += 0.3f;
	myCamAngle.y += 0.5f;
	myCamAngle.z += 0.7f;

	// Rotate the text
	myTextAngle += 1.0f;

	// Get the camera position
	D3DXMATRIX mat1, mat2;

	D3DXMatrixRotationYawPitchRoll( &mat1, D3DXToRadian( myCamAngle.x ), D3DXToRadian( myCamAngle.y ), D3DXToRadian( myCamAngle.z ) );
	D3DXMatrixTranslation( &mat2, 0, 0, 55 );
	D3DXMatrixMultiply( &myCamMat, &mat1, &mat2 );

	// Restart?
	int elapsed = ( int ) ( DXUtil_Timer( TIMER_GETABSOLUTETIME ) - myChangeTime );
	if ( elapsed < 0 ||
		 elapsed > mySetChangeInterval )
	{
		myChangeTime = DXUtil_Timer( TIMER_GETABSOLUTETIME );

		// Restart the computations
		for ( int i = 0; i < MAX_DEVICE_OBJECTS; ++i )
			m_DeviceObjectsArray[ i ].myPointsUsed = 0;
		myFractal.initFractal();
	}

	// Compute more? (has even one of the devices been filled)
	if ( m_pDeviceObjects->myPointsUsed >= myPointsTotal )
		return S_OK;

	// Compute more
	myFractal.compute( ( m_SaverMode != sm_preview ) ? mySetCreateSpeed : 100 );

    return S_OK;
}





//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT Jeresaver::Render()
{
    SetProjectionMatrix( 0.05f, 320.0f );

	// New points?
	const FractalVertexList &vlist = myFractal.getVertices();
	if ( m_pDeviceObjects->myPointsUsed < vlist.size() )
	{
        FractalVertex *v;
        m_pDeviceObjects->m_pVBArray[0]->Lock( 0, 0, (BYTE**)&v, 0 );

		// Prepare
		int max = vlist.size();
		if ( max > myPointsTotal )
			max = myPointsTotal;

		// Copy
		v = &v[ m_pDeviceObjects->myPointsUsed ];
		for ( int i = m_pDeviceObjects->myPointsUsed; i < max; ++i )
		{
			*v++ = vlist[ i ];
		}
		m_pDeviceObjects->myPointsUsed = max;

        m_pDeviceObjects->m_pVBArray[0]->Unlock();
	}

	// Validate the device
    HRESULT hr;
    DWORD dw;
    hr = m_pd3dDevice->ValidateDevice(&dw);

    m_pd3dDevice->SetVertexShader( D3DFVF_FRACTALVERTEX );
    m_pd3dDevice->SetStreamSource( 0, m_pDeviceObjects->m_pVBArray[ 0 ], sizeof( FractalVertex ) );

    // Clear the viewport
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB( 255, 0, 0, 0 ), 1.0f, 0L );

    // Begin the scene 
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
		D3DXMATRIX camMat;

		// Get the display mode
		D3DDISPLAYMODE dispMode;

		m_pd3dDevice->GetDisplayMode( &dispMode );

		float pointSize = ( float ) dispMode.Width / 50.f;
		if ( m_SaverMode == sm_preview )
			pointSize = 5.f;

		// Setup the mode
		m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,			TRUE );
		m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
		m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,		TRUE );
		m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE,		FALSE );

		m_pd3dDevice->SetRenderState( D3DRS_LIGHTING,           FALSE );

		m_pd3dDevice->SetRenderState( D3DRS_CULLMODE,			D3DCULL_CCW );
		m_pd3dDevice->SetRenderState( D3DRS_SHADEMODE,			D3DSHADE_GOURAUD );

		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_DIFFUSE );

/*		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );

		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1 );
		m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );*/

	    m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_ONE );
	    m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ZERO );

		m_pd3dDevice->SetRenderState( D3DRS_POINTSPRITEENABLE, FALSE );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSCALEENABLE,  FALSE );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSIZE,     FtoDW( pointSize /*26.00f*/ ) );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSIZE_MAX, FtoDW( 256.00f ) );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSIZE_MIN, FtoDW( 0.00f ) );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSCALE_A,  FtoDW( 0.00f ) );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSCALE_B,  FtoDW( 0.00f ) );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSCALE_C,  FtoDW( 1.00f ) );

//		m_pd3dDevice->SetTexture( 0, m_pDeviceObjects->m_pTexArray[ 0 ] );

		// Setup the camera
        m_pd3dDevice->SetTransform( D3DTS_VIEW, &myCamMat );

		// Render the vertices
		m_pd3dDevice->DrawPrimitive( D3DPT_POINTLIST, 0, m_pDeviceObjects->myPointsUsed );

		// Render the text
		if ( m_pDeviceObjects->m_pTextMesh != 0 )
		{
			D3DXMATRIX mat1, mat2, mat3;

			// Setup the material / mode
			D3DMATERIAL8 mtrl;
			D3DLIGHT8 light;

			D3DUtil_InitMaterial( mtrl, 0.7f, 0.4f, 0.9f, 0.7f );
			mtrl.Specular.r = mtrl.Specular.g = mtrl.Specular.b = mtrl.Specular.a = 1.f;
			m_pd3dDevice->SetMaterial( &mtrl );

			D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, 1.0f, -1.0f, 10.0f );
			m_pd3dDevice->SetLight( 0, &light );
			m_pd3dDevice->LightEnable( 0, TRUE );

			m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
			m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
			m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
			m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );

			// Setup the camera
			D3DXVECTOR3 camDest = D3DXVECTOR3( 0, 0, 0 );
			D3DXVECTOR3 camPos	= D3DXVECTOR3( 0, myTextCameraZ / -5.f, myTextCameraZ );
			D3DXVECTOR3 camUp	= D3DXVECTOR3( 0, 1, 0 );
			D3DXMatrixLookAtLH( &mat1, &camPos, &camDest, &camUp );

			m_pd3dDevice->SetTransform( D3DTS_VIEW, &mat1 );

			// Setup the world
			D3DXMatrixTranslation( &mat1, -myTextCenter.x, -myTextCenter.y - 1.5f, -myTextCenter.z );
			D3DXMatrixRotationY( &mat2, D3DXToRadian( myTextAngle ) );
			D3DXMatrixMultiply( &mat3, &mat1, &mat2 );

			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat3 );

			// Render
			m_pDeviceObjects->m_pTextMesh->DrawSubset( 0 );

			// Restore the world
			D3DXMatrixIdentity( &mat1 );
			m_pd3dDevice->SetTransform( D3DTS_WORLD, &mat1 );
		}

        // Show frame rate
		m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );

		if ( m_SaverMode == sm_test )
		{
			m_pDeviceObjects->m_pStatsFont->DrawText( 3,  1, D3DCOLOR_ARGB(255,0,0,0), m_strFrameStats );
			m_pDeviceObjects->m_pStatsFont->DrawText( 2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats );

			m_pDeviceObjects->m_pStatsFont->DrawText( 3, 21, D3DCOLOR_ARGB(255,0,0,0), m_strDeviceStats );
			m_pDeviceObjects->m_pStatsFont->DrawText( 2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats );
		}

        // End the scene.
        m_pd3dDevice->EndScene();
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::RestoreDeviceObjects()
{
    if( m_pd3dDevice == NULL )
        return S_OK;
    
    // Set up sensible projection and view matrices
//	D3DXMATRIX  view;
//	D3DXMatrixLookAtLH( &view , &D3DXVECTOR3(0,0,-10) , &D3DXVECTOR3(0,0,0) , &D3DXVECTOR3(0,1,0) );
//	m_pd3dDevice->SetTransform( D3DTS_VIEW , &view );

	// Set some basic renderstates
	m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE , TRUE );
	m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE , FALSE );
	m_pd3dDevice->SetRenderState( D3DRS_LIGHTING,  FALSE );
	m_pd3dDevice->SetRenderState( D3DRS_CULLMODE,  D3DCULL_NONE );
	m_pd3dDevice->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_FLAT );

	if( FAILED( BuildMeshes() ) )
		return E_FAIL;

	if( FAILED( BuildTextures() ) )
		return E_FAIL;

	m_pDeviceObjects->m_pStatsFont = new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
	m_pDeviceObjects->m_pStatsFont->InitDeviceObjects( m_pd3dDevice );
	m_pDeviceObjects->m_pStatsFont->RestoreDeviceObjects();

	return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::InvalidateDeviceObjects()
{
    m_pDeviceObjects->m_pStatsFont->InvalidateDeviceObjects();
    m_pDeviceObjects->m_pStatsFont->DeleteDeviceObjects();
    SAFE_DELETE( m_pDeviceObjects->m_pStatsFont );

    SAFE_RELEASE( m_pDeviceObjects->m_pVBArray[0] );
    SAFE_RELEASE( m_pDeviceObjects->m_pVBArray[1] );
    SAFE_RELEASE( m_pDeviceObjects->m_pVBArray[2] );

    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[0] );
    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[1] );
    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[2] );
    SAFE_RELEASE( m_pDeviceObjects->m_pTexArray[3] );

	SAFE_RELEASE( m_pDeviceObjects->m_pTextMesh );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ReadSettings()
// Desc: 
//-----------------------------------------------------------------------------
VOID Jeresaver::ReadSettings()
{
    ReadRegistry();
}




//-----------------------------------------------------------------------------
// Name: ConfigureDialogProcHelper()
// Desc: 
//-----------------------------------------------------------------------------
INT_PTR CALLBACK ConfigureDialogProcHelper( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    return g_pMyMoireScreensaver->ConfigureDialogProc( hwndDlg, uMsg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: DoConfig()
// Desc: 
//-----------------------------------------------------------------------------
VOID Jeresaver::DoConfig()
{
    DialogBox( NULL, MAKEINTRESOURCE(IDD_CONFIGURE), m_hWndParent, ConfigureDialogProcHelper );
}




//-----------------------------------------------------------------------------
// Name: ConfigureDialogProc()
// Desc: 
//-----------------------------------------------------------------------------
INT_PTR CALLBACK Jeresaver::ConfigureDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    HWND hwndSpeed = GetDlgItem( hwndDlg, IDC_SPEED );
    HWND hwndRotateText = GetDlgItem( hwndDlg, IDC_ROTATETEXT );
    HWND hwndShapeInterval = GetDlgItem( hwndDlg, IDC_SHAPEINTERVAL );
	HWND hwndSpeedText = GetDlgItem( hwndDlg, IDC_SPEEDTEXT );
    TCHAR sz[ 1024 ];

    switch ( uMsg )
    {
    case WM_INITDIALOG:
		WriteRegistry();
		SendMessage( hwndSpeed, TBM_SETRANGE, FALSE, MAKELONG( 100, 10000 ) );
		SendMessage( hwndSpeed, TBM_SETPOS, TRUE, mySetCreateSpeed );

		SetWindowText( hwndRotateText, mySetRotateText.c_str() );

        wsprintf( sz, TEXT( "%d" ), mySetChangeInterval );
        SetWindowText( hwndShapeInterval, sz );

        wsprintf( sz, TEXT( "%d" ), mySetCreateSpeed );
        SetWindowText( hwndSpeedText, sz );
        return TRUE;

	case WM_NOTIFY:
		{
			DWORD val = ( DWORD ) SendMessage( hwndSpeed, TBM_GETPOS, 0, 0 );
	        wsprintf( sz, TEXT( "%d" ), val );
		    SetWindowText( hwndSpeedText, sz );
		}
		return FALSE;

    case WM_COMMAND:
        switch( LOWORD( wParam ) )
        {
        case IDC_ROTATETEXT:
            if( HIWORD( wParam ) == EN_CHANGE )
            {
                GetWindowText( hwndRotateText, sz, 1023 );
				mySetRotateText = sz;
            }
            break;

        case IDC_SHAPEINTERVAL:
            if( HIWORD( wParam ) == EN_CHANGE )
            {
                GetWindowText( hwndShapeInterval, sz, 10 );
                sscanf( sz, TEXT("%d"), &mySetChangeInterval );

				// Need to re-set?
				if ( mySetChangeInterval < 10 || mySetChangeInterval > 99999 )
				{
					mySetChangeInterval = 10;

					wsprintf( sz, TEXT( "%d" ), mySetChangeInterval );
					SetWindowText( hwndShapeInterval, sz );
				}
            }
            break;

        case IDC_SCREENSETTINGS:
            DoScreenSettingsDialog( hwndDlg );
            break;

        case IDOK:
            mySetCreateSpeed = ( DWORD ) SendMessage( hwndSpeed, TBM_GETPOS, 0, 0 );
            WriteRegistry(); // save new settings
            EndDialog( hwndDlg, IDOK );
            break;

        case IDCANCEL:
            ReadRegistry(); // restore previous settings
            EndDialog( hwndDlg, IDCANCEL );
            break;
        }
        return TRUE;
    
    default:
        return FALSE;
    }
}




//-----------------------------------------------------------------------------
// Name: ReadRegistry()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::ReadRegistry()
{
    HKEY hkey;
    DWORD dwType = REG_DWORD;
    DWORD dwLength = sizeof(DWORD);

    DWORD strType = REG_SZ;
	DWORD strLength = 1023;
    TCHAR sz[ 1024 ] = { 0, };

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, m_strRegPath, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {

        RegQueryValueEx( hkey, TEXT("Create Speed"), NULL, &dwType, ( BYTE * ) &mySetCreateSpeed, &dwLength );
        RegQueryValueEx( hkey, TEXT("Change Interval"), NULL, &dwType, ( BYTE * ) &mySetChangeInterval, &dwLength );
		RegQueryValueEx( hkey, TEXT("Rotate Text"), NULL, &strType, ( BYTE * ) &sz, &strLength );

		mySetRotateText = sz;

        ReadScreenSettings( hkey );

        RegCloseKey( hkey );
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: WriteRegistry()
// Desc: 
//-----------------------------------------------------------------------------
HRESULT Jeresaver::WriteRegistry()
{
    HKEY hkey;

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, m_strRegPath, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        RegSetValueEx( hkey, TEXT("Create Speed"), NULL, REG_DWORD, ( BYTE * ) &mySetCreateSpeed, sizeof( DWORD ) );
        RegSetValueEx( hkey, TEXT("Change Interval"), NULL, REG_DWORD, ( BYTE * ) &mySetChangeInterval, sizeof( DWORD ) );
        RegSetValueEx( hkey, TEXT("Rotate Text"), NULL, REG_SZ, ( BYTE * ) mySetRotateText.c_str(), mySetRotateText.length() + 1 );

        WriteScreenSettings( hkey );

        RegCloseKey( hkey );
    }

    return S_OK;
}
