#ifndef __QUATFRAC_H
#define __QUATFRAC_H
#include <bitset>
#include <stack>
#include <vector>

// The index space limits
#define ISPACE_MINX  0
#define ISPACE_MAXX  200

#define ISPACE_MINY  0
#define ISPACE_MAXY  200

#define ISPACE_MINZ  0
#define ISPACE_MAXZ  200

#define ISPACE_SIZEX ( ISPACE_MAXX - ISPACE_MINX + 1 )
#define ISPACE_SIZEY ( ISPACE_MAXY - ISPACE_MINY + 1 )
#define ISPACE_SIZEZ ( ISPACE_MAXZ - ISPACE_MINZ + 1 )

#define ISPACE_SIZEXY  ( ISPACE_SIZEX * ISPACE_SIZEY )
#define ISPACE_SIZEXYZ ( ISPACE_SIZEXY * ISPACE_SIZEZ )

/*
#define ISPACE_MINX  -50
#define ISPACE_MAXX  50

#define ISPACE_MINY  -50
#define ISPACE_MAXY  50

#define ISPACE_MINZ  -50
#define ISPACE_MAXZ  50
*/

// One actual visible quaternion fractal point
struct FractalVertex
{
    D3DXVECTOR3 v;
    D3DCOLOR    color;
};
#define D3DFVF_FRACTALVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)

typedef std::vector< FractalVertex > FractalVertexList;

// Quaternion fractal calculator
class QuaternionFractal
{
public:
	QuaternionFractal();

	// Gets the list of the visible fractal vertices
	inline FractalVertexList &getVertices() { return myVertices; }
	inline const FractalVertexList &getVertices() const { return myVertices; }

	// Initializes the fractal calculator to calculate a random fractal
	void initFractal();

	// Initializes the fractal calculator to calculate a given fractal
	void initFractal( D3DXQUATERNION quat );

	// Computes the fractal n points more
	void compute( int points );

private:
	typedef std::bitset< ISPACE_SIZEXYZ > BitSet;
	typedef std::stack< int > IntStack;
	typedef bool BoolMatrix[ 3 ][ 3 ][ 3 ];

	// List of computed fractal vertices
	FractalVertexList myVertices;

	// List of examined vertices
	BitSet myTestedVertices;
	// List of fractal vertices (tested true)
	BitSet myFractalVertices;

	// List of "to-do" vertices
	IntStack myVertexProcessQueue;

	// The 'c' in the eval func; this defines the fractal to calculate
	D3DXQUATERNION myQuatC;

	// The light direction vector (normalized)
	D3DXVECTOR3 myLightDir;

	// The light/object color (0-1)
	D3DXVECTOR3 myLightCol;

	// Tests if the given index point is in the fractal or not
	bool testPoint( int x, int y, int z );

	// Calculates a shading value (0-1) when given an adjancency matrix [x][y][z]
	float calcShade( BoolMatrix &matrix );
};

////


// Converts an (x,y,z) int coordinate to a table index
inline int coordToIndex( int x, int y, int z )
{
	return ( x - ISPACE_MINX ) +
		   ( y - ISPACE_MINY ) * ISPACE_SIZEX +
		   ( z - ISPACE_MINZ ) * ISPACE_SIZEXY;
}

// Converts a table index to (x,y,z) int coordinates
inline void indexToCoord( int idx, int &x, int &y, int &z )
{
	x = ISPACE_MINX + ( idx % ISPACE_SIZEX );
	idx /= ISPACE_SIZEX;

	y = ISPACE_MINY + ( idx % ISPACE_SIZEY );
	idx /= ISPACE_SIZEY;

	z = ISPACE_MINZ + idx;
}

#endif
